@@ -43,13 +43,23 @@ Then to use it..
 """
 
 __author__ = "dbr/Ben"
-__version__ = "1.9"
+__version__ = "1.10"
 
+import sys
 import logging
 import warnings
 
 from tvdb_exceptions import tvdb_userabort
 
+
+IS_PY2 = sys.version_info[0] == 2
+
+if IS_PY2:
+    user_input = raw_input
+else:
+    user_input = input
+
+
 def log():
     return logging.getLogger(__name__)
 
@@ -80,7 +90,7 @@ class ConsoleUI(BaseUI):
         else:
             toshow = allSeries
 
-        print "TVDB Search Results:"
+        print("TVDB Search Results:")
         for i, cshow in enumerate(toshow):
             i_show = i + 1 # Start at more human readable number 1 (not 0)
             log().debug('Showing allSeries[%s], series %s)' % (i_show, allSeries[i]['seriesname']))
@@ -89,31 +99,35 @@ class ConsoleUI(BaseUI):
             else:
                 extra = ""
 
-            print "%s -> %s [%s] # http://thetvdb.com/?tab=series&id=%s&lid=%s%s" % (
+            output = "%s -> %s [%s] # http://thetvdb.com/?tab=series&id=%s&lid=%s%s" % (
                 i_show,
-                cshow['seriesname'].encode("UTF-8", "ignore"),
-                cshow['language'].encode("UTF-8", "ignore"),
+                cshow['seriesname'],
+                cshow['language'],
                 str(cshow['id']),
                 cshow['lid'],
                 extra
             )
+            if IS_PY2:
+                print(output.encode("UTF-8", "ignore"))
+            else:
+                print(output)
 
     def selectSeries(self, allSeries):
         self._displaySeries(allSeries)
 
         if len(allSeries) == 1:
             # Single result, return it!
-            print "Automatically selecting only result"
+            print("Automatically selecting only result")
             return allSeries[0]
 
         if self.config['select_first'] is True:
-            print "Automatically returning first search result"
+            print("Automatically returning first search result")
             return allSeries[0]
 
         while True: # return breaks this loop
             try:
-                print "Enter choice (first number, return for default, 'all', ? for help):"
-                ans = raw_input()
+                print("Enter choice (first number, return for default, 'all', ? for help):")
+                ans = user_input()
             except KeyboardInterrupt:
                 raise tvdb_userabort("User aborted (^c keyboard interupt)")
             except EOFError:
@@ -131,13 +145,13 @@ class ConsoleUI(BaseUI):
                     log().debug('Got quit command (q)')
                     raise tvdb_userabort("User aborted ('q' quit command)")
                 elif ans == "?":
-                    print "## Help"
-                    print "# Enter the number that corresponds to the correct show."
-                    print "# a - display all results"
-                    print "# all - display all results"
-                    print "# ? - this help"
-                    print "# q - abort tvnamer"
-                    print "# Press return with no input to select first result"
+                    print("## Help")
+                    print("# Enter the number that corresponds to the correct show.")
+                    print("# a - display all results")
+                    print("# all - display all results")
+                    print("# ? - this help")
+                    print("# q - abort tvnamer")
+                    print("# Press return with no input to select first result")
                 elif ans.lower() in ["a", "all"]:
                     self._displaySeries(allSeries, limit = None)
                 else:
@@ -148,6 +162,6 @@ class ConsoleUI(BaseUI):
                     return allSeries[selected_id]
                 except IndexError:
                     log().debug('Invalid show number entered!')
-                    print "Invalid number (%s) selected!"
+                    print("Invalid number (%s) selected!")
                     self._displaySeries(allSeries)
 
